//ChangerLangue_509f.js
// 25.04.15 / Hans Haesler, Chtelard 52, CH-1018 Lausanne / <hsa@ringier.ch>
//DESCRIPTION:Changer la langue des documents ouverts 
//ou des documents dans un dossier slectionn

// les listes des langues disponibles
var allLang = app.languagesWithVendors;
var lngNameList = app.languagesWithVendors.everyItem().name;
var locLangList = app.languagesWithVendors.everyItem().name;
// trier
locLangList.sort(
	function(a, b) {
		if (a.toLowerCase() < b.toLowerCase()) return -1;
		if (a.toLowerCase() > b.toLowerCase()) return 1;
		return 0;
	}
);
// le nom d'"Aucune langue"
var noLang = locLangList[0];

// lire le fichier qui contient la langue applique la dernire fois
var curLang = 0;
if ($.os.slice(0,7) == "Windows") {
	var tempFolder = Folder.temp;
	var f = File(tempFolder + "/curLang.txt");
}
else {
	var f = new File("~/Library/Preferences/curLang.txt");
}
var fileIsOpen = f.open("r");
if (fileIsOpen) {
	try {
		curLang = (f.read())*1;
		f.close();
	}
	catch (e) {
		try {
			f.close();
		}
		catch (e) {
		}
	}
}

// effacer des dialogues qui sont ventuellement rests dans la mmoire
try {
	app.dialogs.everyItem().destroy();
}
catch (e) {
}

// activer l'affichage des dialogues
app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;

// tester si un document est ouvert (sinon: montrer un dilaogue)
var nDocs = app.documents.length;
if (nDocs == 0) {
	var folderObj = File (Folder.selectDialog("Slectionner le dossier contenant les documents InDesign."));
	var testObj = "" + folderObj;
	if (testObj.match("null")) {
		exit();
	}
	// appeler la fonction qui montre le dialogue avec le choix des langues
	var dialogIndex = askUser();
	var newLang = getLang(lngNameList, locLangList, dialogIndex);
	// dsactiver les messages ventuels causs par des polices et des images manquants
	app.scriptPreferences.userInteractionLevel = UserInteractionLevels.neverInteract;
	// dresser une liste du contanu du dossier
	var allFiles = folderObj.getFiles();
	
	// une boucle  travers les fichiers
	var curName;
	for (var i=0; i<allFiles.length; i++) {
		curName = allFiles[i].name;
		if (curName.match(".indd")) {
			try {
				// ouvrir le document actuel
				app.open(File (allFiles[i]));
				changeLanguage(newLang);
				app.activeDocument.close(SaveOptions.yes);
			}
			catch (e) {
			}
		}
	}
	// activer les messages
	app.scriptPreferences.userInteractionLevel = UserInteractionLevels.interactWithAll;
}
else {
	var dialogIndex = askUser();
	var newLang = getLang(lngNameList, locLangList, dialogIndex);
	var docRef = new Array();
	for (var i=0; i<nDocs; i++) {
		docRef.push(app.documents[i].name);
	}
	var okay = true;
	var docName;
	for (var i=0; i<docRef.length; i++) {
		app.activeDocument = app.documents.item(docRef[i]);
		docName = app.activeDocument.name;
		if (nDocs>1 && i>0) {
			var okay = confirm ("Changer la langue de ce document?\n\nConfirmer par un clic sur \"Yes\".");
		}
		if (okay) {
			changeLanguage(newLang);
		}
	}
	app.activeDocument = app.documents.item(docRef[0]);
}

// annoncer la fin de l'action
var dlog = new Window("palette");
dlog.size = [320,100];
dlog.add("statictext", [80, 100, 320, 116], "C'est fait.");
dlog.show();
// une petite pause
$.sleep(1500);
// fermer la fentre
dlog.close();


////////////////////////////// les fonctions  /////////////////////////////////////////////////////

// la fonction qui permet de slectionner la nouvelle langue //////////////////////////////////////
function askUser() {
	// prparer le dialogue
	var aDialog = app.dialogs.add({name:"Changer la langue", canCancel:true});
	with (aDialog) {
		with (dialogColumns.add()) {
			with (dialogRows.add()) {
				staticTexts.add({staticLabel:"La nouvelle langue:", minWidth:80});
				staticTexts.add({staticLabel:"", minWidth:20});
			}
			with (borderPanels.add()) {
				with (dialogColumns.add()) {
					var langName = dropdowns.add({stringList:locLangList, selectedIndex:curLang, minWidth:130});
				}
			}
			with (dialogColumns.add()) {
				staticTexts.add({staticLabel:"", minWidth:4});
			}
		}
	}
	
	// afficher le dialogue et mmoriser le choix de l'utilisateur
	if (aDialog.show() == true) {
		var dialogIndex = langName.selectedIndex;
		aDialog.destroy();
	}
	else {
		aDialog.destroy();
		exit();
	}
	
	// crire le numro de la langue dans le fichier
	if ($.os.slice(0,7) == "Windows") {
		var tempFolder = Folder.temp;
		var f = File(tempFolder + "/curLang.txt");
	}
	else {
		var f = new File("~/Library/Preferences/curLang.txt");
	}
	var fileIsOpen = f.open("w");
	if (fileIsOpen) {
		try {
			f.write(dialogIndex);
			f.close();
		}
		catch (e) {
			try {
				f.close();
			}
			catch (e) {
			}
		}
	}
	return dialogIndex;
}

// la fonction qui sert  dfinir la langue ///////////////////////////////////////////////////////
function getLang(lngNameList, locLangList, dialogIndex) {
	for (var i=0; i<lngNameList.length; i++) {
		if (lngNameList[i] == locLangList[dialogIndex]) {
			return i;
		}
	}
}

// la fonction qui procde au changement de la langue /////////////////////////////////////////////
function changeLanguage(newLang) {
	// changer la langue des styles de paragraphes
	var curLang;
	var nParaStyles = app.documents[0].paragraphStyles.length;
	for (var i=1; i<nParaStyles; i++) {
		try {
			curLang = app.documents[0].paragraphStyles[i].appliedLanguage.name;
			if (curLang != noLang) {
				app.documents[0].paragraphStyles[i].appliedLanguage = allLang[newLang];
			}
		}
		catch (e) {
		}
	}
	
	// changer la langue des styles de paragraphes des groupes
	var curGroups = app.documents[0].paragraphStyleGroups;
	var nParaStyleGroups = curGroups.length;
	for (var i=0; i<nParaStyleGroups; i++) {
		var curGroup = curGroups[i];
		var curParaStyles = curGroup.allParagraphStyles;
		var nParaStyles = curParaStyles.length;
		for (var k=0; k<nParaStyles; k++) {
			try {
				curLang = curParaStyles[k].appliedLanguage.name;
				if (curLang != noLang) {
					curParaStyles[k].appliedLanguage = allLang[newLang];
				}
			}
			catch (e) {
			}
		}
	}
	
	// changer la langue des styles de caractres
	nCharStyles = app.documents[0].characterStyles.length;
	for (var i=1; i<nCharStyles; i++) {
		try {
			curLang = app.documents[0].characterStyles[i].appliedLanguage.name;
			if (curLang != noLang) {
				app.documents[0].characterStyles[i].appliedLanguage = allLang[newLang];
			}
		}
		catch (e) {
		}
	}
	
	// changer la langue des styles de caractres des groupes
	curGroups = app.documents[0].characterStyleGroups;
	var nCharStyleGroups = curGroups.length;
	for (var i=0; i<nCharStyleGroups; i++) {
		var curGroup = curGroups[i];
		var curCharStyles = curGroup.allCharacterStyles;
		var nCharStyles = curCharStyles.length;
		for (var k=0; k<nCharStyles; k++) {
			try {
				curLang = curCharStyles[k].appliedLanguage.name;
				if (curLang != noLang) {
					curCharStyles[k].appliedLanguage = allLang[newLang];
				}
			}
			catch (e) {
			}
		}
	}
	
	// modifier la langue des textes, afin de ne pas oublier ceux sans feuille de style
	app.documents[0].stories.everyItem().appliedLanguage = allLang[newLang];
	
	// traiter galement les tableaux...
	try {
		app.documents[0].stories.everyItem().tables.everyItem().cells.everyItem().texts[0].appliedLanguage = allLang[newLang];
	}
	catch (e) {
	}
}